﻿package no.ekse.productball 
{
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.display.Stage;
	import flash.geom.Point;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFieldAutoSize;
	import no.ekse.common.LinkedList;
	
	import no.ekse.mooses.MoosesPlayer;
	/**
	 * ...
	 * @author Magnus Ekse
	 */
	public class ProductField extends Sprite
	{
		private var leftPlayer:ProductPlayer;
		private var rightPlayer:ProductPlayer;
			
			
		private var leftFloor:Sprite = new Sprite();
		private var leftWall:Sprite = new Sprite();
		private var leftBoundary:Number;
		private var leftVoteBar:VoteBar = new VoteBar();
		
		private var rightFloor:Sprite = new Sprite();
		private var rightWall:Sprite = new Sprite();
		private var rightBoundary:Number;
		private var rightVoteBar:VoteBar = new VoteBar();
		
		private var net:Sprite = new Sprite();
		private var netPosition:Number;
		private var netHeight:Number;
		private var netWidth:Number;
		
		private var courtPosition:Number;
		
		private var leftPlayerList:TextField;
		private var rightPlayerList:TextField;
		
		private var pointboard:TextField;
		
		private var ball:ProductBall;
		
		private var numIdleUpdates:int = 300;
		
		private var leftKeyHandler:KeyController;
		private var rightKeyHandler:KeyController;
		
		private var main:Main;
		
		[Embed(source='res/background.png')]
		private static const backgroundGfx:Class;
		
		private var backgroundSprite:Bitmap;
		
		public static const screenWidth:int = 1024;
		public static const screenHeight:int = 768;
		
		public function ProductField(main:Main) 
		{
			this.main = main;
			backgroundSprite = new ProductField.backgroundGfx();
			
			leftPlayer = new ProductPlayer(true, leftVoteBar);
			rightPlayer = new ProductPlayer(false, rightVoteBar);
			ball = new ProductBall();
			
			leftKeyHandler = new KeyController(leftPlayer, true, main);
			rightKeyHandler = new KeyController(rightPlayer, false, main);
			
			addChild(backgroundSprite);
			addChild(leftPlayer);
			addChild(rightPlayer);
			addChild(leftKeyHandler);
			addChild(rightKeyHandler);
			addChild(ball);
			
			leftBoundary = ball.getRadius();
			rightBoundary = screenWidth - ball.getRadius();
			netPosition = screenWidth / 2;
			netHeight = 250;
			netWidth = 75;
			
			courtPosition = screenHeight - 60;
			
			leftFloor.graphics.beginFill(0x800000);
			leftFloor.graphics.drawRect(leftBoundary, courtPosition, netPosition - leftBoundary, 100);
			leftFloor.alpha = 0.75;
			leftFloor.graphics.endFill();
			
			//addChild(leftFloor);
			
			rightFloor.graphics.beginFill(0x000080);
			rightFloor.graphics.drawRect(netPosition, courtPosition, rightBoundary - netPosition, 100);
			rightFloor.alpha = 0.75;
			rightFloor.graphics.endFill();
			
			//addChild(rightFloor);
			
			
			leftWall.graphics.beginFill(0x008000);
			leftWall.graphics.drawRect(0, 0, leftBoundary, courtPosition);
			leftWall.alpha = 0.75;
			leftWall.graphics.endFill();
			
			//addChild(leftWall);
			
			
			rightWall.graphics.beginFill(0x008000);
			rightWall.graphics.drawRect(rightBoundary, 0, 100, courtPosition);
			rightWall.alpha = 0.75;
			rightWall.graphics.endFill();
			
			//addChild(rightWall);
			
			net.graphics.beginFill(0x008000);
			net.graphics.drawRect(netPosition - netWidth / 2, courtPosition - netHeight, netWidth, netHeight);
			net.alpha = 0.75;
			net.graphics.endFill();
			
			//addChild(net);
			
			pointboard = new TextField();
			pointboard.x = screenWidth / 2 - 50;
			pointboard.y = 50;
			pointboard.autoSize = TextFieldAutoSize.CENTER;
			
			var scoreFormat:TextFormat = new TextFormat();
            scoreFormat.font = "Verdana";
            scoreFormat.color = 0xFFFFFF;
            scoreFormat.size = 30;
            
            pointboard.defaultTextFormat = scoreFormat;
			
			addChild(pointboard);
			
			pointboard.text = "Score\n0 : 0";
			
			
			
			
			leftPlayerList = new TextField();
			leftPlayerList.x = 50;
			leftPlayerList.y = 50;
			leftPlayerList.autoSize = TextFieldAutoSize.LEFT;
			leftPlayerList.background = true;
			leftPlayerList.backgroundColor = 0x000040;
			
			rightPlayerList = new TextField();
			rightPlayerList.x = screenWidth - 150;
			rightPlayerList.y = 50;
			rightPlayerList.autoSize = TextFieldAutoSize.RIGHT;
			rightPlayerList.background = true;
			rightPlayerList.backgroundColor = 0x400000;
			
			var listFormat:TextFormat = new TextFormat();
            listFormat.font = "Verdana";
            listFormat.color = 0xFFFFFF;
            listFormat.size = 30;
			
			leftPlayerList.defaultTextFormat = listFormat;
			rightPlayerList.defaultTextFormat = listFormat;
			
			addChild(leftPlayerList);
			addChild(rightPlayerList);
			
			
			
			
			leftVoteBar.y = 738;
			leftVoteBar.x = 50;
			
			rightVoteBar.x = 100 + (1024 - 100) / 2;
			rightVoteBar.y = 738;
			
			addChild( leftVoteBar );
			addChild( rightVoteBar );
			
			resetBall(false);
			
			
			
			
			
			
			var players:Array = main.getPlayerList();
			var i:int = 0;
			
			for (i = 0; i < players.length; i++) {
				
				var v:Voter = new Voter((players[i] as MoosesPlayer).getPlayerName());
				(players[i] as MoosesPlayer).attachUnit(v);
				
				if (i % 2 == 0) {
					trace("new voter " + (players[i] as MoosesPlayer).getPlayerName());
					leftPlayer.addVoter(v);
				} else {
					trace("new voter " + (players[i] as MoosesPlayer).getPlayerName());
					rightPlayer.addVoter(v);
				}
			}
			
			leftPlayerList.text = "LEFT TEAM:\n" + leftPlayer.getVoters();
			rightPlayerList.text = "RIGHT TEAM:\n" + rightPlayer.getVoters();
			
			
			
			
		}
		
		public function resetBall(fromLeftPlayer:Boolean):void
		{
			if (fromLeftPlayer) {
				ball.x = 100;
				ball.y = 400;
				ball.setVelocity(new Point(4, -8));
			} else {
				ball.x = screenWidth - 100;
				ball.y = 400;
				ball.setVelocity(new Point(-4, -8));
			}
			
		}
		
		public function updatePointboard():void
		{
			pointboard.text = "Score\n" + leftPlayer.getScore() + " : " + rightPlayer.getScore();
		}
		
		public function update():void
		{
			if (numIdleUpdates > 0) {
				numIdleUpdates--;
				
				return;
			} else if (numIdleUpdates == 0) {
				removeChild(leftPlayerList);
				removeChild(rightPlayerList);
				numIdleUpdates--;
			}
			
			var bounceForce:Point;
			var speedForce:Point;
			
			ball.getVelocity().y += 0.1;
			
			leftPlayer.update();
			rightPlayer.update();
			
			if (leftPlayer.y < courtPosition - leftPlayer.getPlayerHeight()) {
				leftPlayer.getVelocity().y += 1;	
			} else {
				leftPlayer.y = courtPosition - leftPlayer.getPlayerHeight();
				leftPlayer.getVelocity().y = 0;
			}
			if (rightPlayer.y < courtPosition - rightPlayer.getPlayerHeight()) {
				rightPlayer.getVelocity().y += 1;	
			} else {
				rightPlayer.y = courtPosition - rightPlayer.getPlayerHeight();
				rightPlayer.getVelocity().y = 0;
			}
			
			var distToLeftPlayer:Number = Math.sqrt( Math.pow(ball.x - leftPlayer.x, 2) + Math.pow(ball.y - leftPlayer.y, 2) );
			var distToRightPlayer:Number = Math.sqrt( Math.pow(ball.x - rightPlayer.x, 2) + Math.pow(ball.y - rightPlayer.y, 2) );
			
			if (ball.getVelocity().y > 0 && ball.getRadius() + leftPlayer.getRadius() > distToLeftPlayer) {
				bounceForce = new Point(ball.x - leftPlayer.x, ball.y - leftPlayer.y);
				speedForce = new Point(leftPlayer.getVelocity().x, leftPlayer.getVelocity().y);
			}
			
			if (ball.getVelocity().y > 0 && ball.getRadius() + rightPlayer.getRadius() > distToRightPlayer) {
				bounceForce = new Point(ball.x - rightPlayer.x, ball.y - rightPlayer.y);
				speedForce = new Point(rightPlayer.getVelocity().x, rightPlayer.getVelocity().y);
			}
			
			if (ball.getVelocity().y > 0 && (ball.getRadius() + rightPlayer.getRadius() > distToRightPlayer || ball.getRadius() + leftPlayer.getRadius() > distToLeftPlayer)) {
				bounceForce.normalize(7);
				speedForce.normalize(1);
				ball.setVelocity(bounceForce.add(speedForce));
			}
			
			ball.x += ball.getVelocity().x;
			ball.y += ball.getVelocity().y;
			
			if (ball.getVelocity().x > 0 && ball.x > netPosition - netWidth / 2 && ball.x < netPosition &&  ball.y > courtPosition - netHeight + 20) {
				ball.getVelocity().x = -ball.getVelocity().x;
			}
			
			if (ball.getVelocity().x < 0 && ball.x > netPosition && ball.x < netPosition + netWidth / 2 &&  ball.y > courtPosition - netHeight + 20) {
				ball.getVelocity().x = -ball.getVelocity().x;
			}
			
			if (ball.getVelocity().y > 0 && ball.x >= netPosition - netWidth / 2 && ball.x <= netPosition + netWidth / 2 &&  ball.y > courtPosition - netHeight && ball.y > courtPosition - netHeight + 20) {
				ball.getVelocity().y = -ball.getVelocity().y * 0.5;
			}
			
			if (ball.x > screenWidth - ball.getRadius()) {
				ball.x = screenWidth - ball.getRadius();
				ball.getVelocity().x = -ball.getVelocity().x;
			}
			if (ball.x < ball.getRadius()) {
				ball.x = ball.getRadius();
				ball.getVelocity().x = -ball.getVelocity().x;
			}
			
			if (ball.y > courtPosition) {
				if (ball.x < netPosition) {
					rightPlayer.addScore(1);
					resetBall(false);
				} else {
					leftPlayer.addScore(1);
					resetBall(true);
				}
				updatePointboard();
			}
			
			if (ball.getVelocity().y > 5) {
				ball.getVelocity().y = 5;
			}
			
			
			if (leftPlayer.x < leftBoundary) {
				leftPlayer.x = leftBoundary;
			}
			if (rightPlayer.x < netPosition + netWidth / 2) {
				rightPlayer.x = netPosition + netWidth / 2;
			}
			if (leftPlayer.x > netPosition - netWidth / 2) {
				leftPlayer.x = netPosition - netWidth / 2;
			}
			if (rightPlayer.x > rightBoundary) {
				rightPlayer.x = rightBoundary;
			}
			
		}
		
	}
	
}