﻿package  no.ekse.mooses
{
	import flash.events.EventDispatcher;
	import flash.net.Socket;
	import flash.utils.ByteArray;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	
	
	/**
	 * The main module which maintains a connection between Flash and the MOOSES framework.
	 * @author Magnus Ekse
	 */
	public class MoosesFrameworkClient extends EventDispatcher
	{
		
		public static const KEY_STATE_UP:int = 1;
		public static const KEY_STATE_DOWN:int = 0;
		public static const KEY_STATE_SPECIAL:int = 2;

		public static const KEY_0:int = 0;
		public static const KEY_1:int = 1;
		public static const KEY_2:int = 2;
		public static const KEY_3:int = 3;
		public static const KEY_4:int = 4;
		public static const KEY_5:int = 5;
		public static const KEY_6:int = 6;
		public static const KEY_7:int = 7;
		public static const KEY_8:int = 8;
		public static const KEY_9:int = 9;
		public static const KEY_STAR:int = 10;
		public static const KEY_HASH:int = 11;
		public static const KEY_A:int = 12;
		public static const KEY_B:int = 13;
		
		private var sck:Socket = new Socket();
		
		private var remoteAddress:String;
		private var remotePort:int;
		
		private var hasReceivedStartupSignal:Boolean = false;
		
		
		public function MoosesFrameworkClient() {
			
			trace("Mooses: Initializing framework client");
			
			remoteAddress = "localhost";
			remotePort = 1900;
			
			
			trace("Mooses: Framework client ready");
			
		}
		
		public function initSocket(a:String, p:int):void {
			
			trace("Mooses: Connecting to socket: " + a + ":" + p);
			dispatchEvent(new MoosesConnectionEvent(MoosesConnectionEvent.MOOSES_ATTEMPTCONNECT, false, false));
			
			sck = new Socket(a, p);
			
			sck.addEventListener(Event.CONNECT, socketConnect);
			sck.addEventListener(Event.CLOSE, socketClose);
			sck.addEventListener(IOErrorEvent.IO_ERROR, socketIOError);
			sck.addEventListener(ProgressEvent.SOCKET_DATA, socketReceive);
			
		}
		
		
		public function connectedToFramework():Boolean {
			
			trace("Mooses: Checking if connected to framework");
			
			if (hasReceivedStartupSignal) {
				trace("Mooses: Has received startup signal");
				
				if (sck.connected) {
					trace("Mooses: Socket is connected, connection OK");
					
					return true;
				}
			}
			
			trace("Mooses: Not connected to framework");
			
			return false;
			
		}
		
		
		public function connect():void {
			
			trace("Mooses: Attempting to connect to " + remoteAddress + ":" + remotePort);
			
			hasReceivedStartupSignal = false;
			initSocket(remoteAddress, remotePort);
			
		}
		
		
		public function connectTo(address:String, port:int):void {
			
			trace("Mooses: Attempting to connect to " + address + ":" + port);
			
			hasReceivedStartupSignal = false;
			initSocket(address, port);
			
		}
		
		
		private function socketConnect(e:Event):void {
			
			trace("Mooses: Socket connected");
			
		}
		
		
		private function socketClose(e:Event):void {
			
			trace("Mooses: Socket closed");
				dispatchEvent(new MoosesConnectionEvent(MoosesConnectionEvent.MOOSES_DISCONNECTED, false, false));
			if (hasReceivedStartupSignal) {
			}
		}
		
		/*
		 * Will buzz the controller of the specified player for a
		 * duration specified in miliseconds
		 * 
		 * How the buzz is implemented is up to the controller, a mobile
		 * phone will vibrate.
		 */
		public function buzzPlayer(playerId:int, buzzDuration:int):void {
			
			if (hasReceivedStartupSignal && sck.connected && playerId < 9000) {
				
				sck.writeInt(1020);
				sck.writeInt(playerId);
				sck.writeInt(buzzDuration);
				
				sck.flush();
			}
	
		}
		
		/*
		 * Update the amount of points the player has
		 * 
		 */
		public function updatePlayerScore(playerId:int, score:int):void {
			
			if (hasReceivedStartupSignal && sck.connected && playerId < 9000) {
				
				trace("Mooses: Send: Type: " + 1030);
				sck.writeInt(1030);
				
				trace("Mooses: Send: Player: " + playerId);
				sck.writeInt(playerId);
				
				trace("Mooses: Send: Score: " + score);
				sck.writeInt(playerId);
				
			}
		}
		
		/*
		 * Sends a message to the player. How this message is interpreted is up to the client script.
		 * This function is therefore as general as possible to allow maximum usability. Shorter and
		 * more concrete functions to update the client should be made on a per-game basis.
		 * 
		 * Only two types of data is supported; integer and string.
		 */
		public function sendMessageToPlayer(playerId:int, messageType:int, messageData:Array):void {
			
			if (hasReceivedStartupSignal && sck.connected && playerId < 9000) {
				
				trace("Mooses: Send: Type: " + messageType);
				sck.writeInt(messageType);
				
				trace("Mooses: Send: Player: " + playerId);
				sck.writeInt(playerId);
				
				//sending the number of content units, not necessarily the size
				trace("Mooses: Send: NumPax: " + messageData.length);
				sck.writeByte(messageData.length);
				
				for (var i:int = 0; i < messageData.length; i++) {
					
					if (messageData[i] is String) {
						//this part is guesswork
						trace("Mooses: Send: Pax " + i + ": String: " + (messageData[i] as String));
						sck.writeByte(5); // 5 -> next data is string
						sck.writeUTF(messageData[i] as String);
						sck.writeByte(0);
						
					} else if (messageData[i] is int) {
						trace("Mooses: Send: Pax " + i + ": Int: " + (messageData[i] as int));
						sck.writeByte(2); // 2 -> next data is int
						sck.writeInt(messageData[i] as int);
						
					}
					
				}
				
				sck.flush();
				
				trace("Mooses: Send: Done");
				
			}
	
		}
		
		private function flushRemainingBytes():void {
			
			var b:ByteArray = new ByteArray();
			sck.readBytes(b, 0, sck.bytesAvailable);
			trace(b);
			
		}
		
		private function socketReceive(e:ProgressEvent):void {
			
			trace("Mooses: Socket received data, bytes in buffer = " + sck.bytesAvailable);
			
			var bytesActuallyRead:int = 0;
			
			if (sck.bytesAvailable < 8) {
				trace("Mooses: Too little data in the incoming buffer, skipping this and waiting for more");
				
				return;
			}
			
			if (!hasReceivedStartupSignal) {
				
				//Expected data in the buffer is two int values.
				trace("Mooses: Startup signal recieved, " + sck.readInt() + " . " + sck.readInt());
				
				//Should be good to go now.
				hasReceivedStartupSignal = true;
				dispatchEvent(new MoosesConnectionEvent(MoosesConnectionEvent.MOOSES_CONNECTED, false, false));
				
			} 
			if (hasReceivedStartupSignal) {
				
				while (sck.bytesAvailable >= 12) {
				
					trace("bytes available " + sck.bytesAvailable);
					
					if (sck.bytesAvailable < 4) { flushRemainingBytes(); return; }
					var packetId:int = sck.readInt();
					trace("packetId " + packetId);
					
					if (sck.bytesAvailable < 4) { flushRemainingBytes(); return; }
					var playerId:int = sck.readInt();
					trace("playerId " + playerId);
					//Get the size of the packet (without the header).
					
					if (sck.bytesAvailable < 4) { flushRemainingBytes(); return; }
					var packetSize:int = sck.readInt();
					trace("packetSize " + packetSize);
					
					bytesActuallyRead = sck.bytesAvailable;
					
					trace("Mooses: Packet received: " + packetId + ", " + playerId + ", " + packetSize);
					
					if (packetId == 1000) {
						//Server started
						
					} else if (packetId == 1010) {
						//Player joins
						var playerName:String = sck.readUTF();
						
						trace("Mooses: Player joined, id = " + playerId + ", name = " + playerName);
						
						trace(sck.readByte());
						
						dispatchEvent(new MoosesPlayerEvent(MoosesPlayerEvent.MOOSES_PLAYER_JOIN, playerId, playerName, false, false));
						
					} else if (packetId == 1011) {
						//Player quits.
						
						trace("Mooses: Player quits, id = " + playerId);
						
						dispatchEvent(new MoosesPlayerEvent(MoosesPlayerEvent.MOOSES_PLAYER_QUIT, playerId, "", false, false));
						
					} else if (packetId == 40) {
						//Special button. TBD.
						trace("Mooses: Special button pressed, 40");
						
						dispatchEvent(new MoosesKeyEvent(MoosesKeyEvent.MOOSES_KEY_EVENT, playerId, 10, 2, false, false));
						
					} else if (packetId == 80) {
						//Special button. TBD.
						trace("Mooses: Special button pressed, 80");
						
						dispatchEvent(new MoosesKeyEvent(MoosesKeyEvent.MOOSES_KEY_EVENT, playerId, 11, 2, false, false));
						
					} else if (packetId == 0) {
						//Movement key pressed
						
						if (sck.bytesAvailable >= 5) {
							
							if (sck.bytesAvailable < 4) { flushRemainingBytes(); return; }
							
							var keyId:int = sck.readInt();
							
							if (sck.bytesAvailable < 1) { flushRemainingBytes(); return; }
							
							var keyState:Boolean = sck.readBoolean();
							
							trace("Mooses: Movement key packet, key = " + keyId + ", status = " + keyState);
							
							/*  a   b	|  12    13
							 *  1 2 3	|  01 02 03
							 *  4 5 6	|  04 05 06
							 *  7 8 9	|  07 08 09
							 *  * 0 #	|  10 00 11
							 */
							
							//The key ids sent by MOOSES are not particulary logical and I suspect them to be
							//dependent on the client. This section _might_ have to be edited to conform to
							//different clients for different games.
							if (keyId == 100)
							{
								keyId = KEY_4;
							} else if (keyId == 102)
							{
								keyId = KEY_2;
							} else if (keyId == 101)
							{
								keyId = KEY_6;
							} else if (keyId == 103)
							{
								keyId = KEY_8;
							}
							
							dispatchEvent(new MoosesKeyEvent(MoosesKeyEvent.MOOSES_KEY_EVENT, playerId, keyId, keyState == 0 ? 1 : 0, false, false));
						} else {
							
							trace("Mooses Warning: Hmm, erroneous number of bytes left in buffer: " + sck.bytesAvailable);
							trace("Mooses Warning: Peeking at mystery byte: " + sck.readByte());
							flushRemainingBytes();
						}
					} else {
						
						flushRemainingBytes();
					}
					
					bytesActuallyRead = bytesActuallyRead - sck.bytesAvailable;
					trace("Bytes processed this run:" + (bytesActuallyRead));
				}
				
			}
			
			trace("Mooses: Bytes in buffer after handling: " + sck.bytesAvailable);
			
			flushRemainingBytes();
			
		}
		
		
		private function socketIOError(e:IOErrorEvent):void {
			
			trace("Mooses: Socket IO error");
			dispatchEvent(new MoosesConnectionEvent(MoosesConnectionEvent.MOOSES_ERRORCONNECT, false, false));
		}
	}
	
}