﻿package  no.ekse.elk
{
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.geom.Point;
	import flash.text.TextField;
	import flash.utils.getTimer;
	import no.ekse.common.ConcreteLinkedList;
	import no.ekse.common.LinkedList;
	import no.ekse.elk.buff.IBuff;
	import no.ekse.mooses.MoosesPlayer;
	import no.ekse.mooses.MoosesFrameworkClient;
	
	import no.ekse.mooses.MoosesUnit;
	import no.ekse.common.LinkedListNode;
	import no.ekse.elk.buff.IBuffable;
	
	/**
	 * ...
	 * @author Magnus Ekse
	 */
	public class Attacker extends Sprite implements MoosesUnit, IBuffable, LinkedListNode
	{
		[Embed(source='res/player2.png')]
		private static const GfxAttacker:Class;
		[Embed(source='res/player2_w1.png')]
		private static const GfxAttackerWalk1:Class;
		[Embed(source='res/player2_w2.png')]
		private static const GfxAttackerWalk2:Class;
		
		
		[Embed(source='res/player2dead.png')]
		private static const GfxAttackerDead:Class;
		
		[Embed(source='res/attackerarrow.png')]
		private static const GfxAttackerArrow:Class;
		
		
		private var gfx:Bitmap;

		private var plate:Nameplate;
		
		private var world:World = null;
		private var player:MoosesPlayer = null;
		
		private var direction:int = 0;
		private var velocity:Point = new Point(0, 0);
		private var haste:Number = 0;
		
		private var health:int = 100;
		private var alive:Boolean = true;
		
		private var directionChange:Number = 0;
		
		private var _prevNode:LinkedListNode;
		private var _nextNode:LinkedListNode;
		
		private var timeOfDeath:Number;
		private var timeOfRemoval:Number;
		
		private var buffs:LinkedList = new ConcreteLinkedList();
		
		private var lastFrameSwap:Number = 0;
		private var lastFrame:int = 0;
		
		public var removed:Boolean = false;
		
		
		private var angle:Number = 0;
		private var angleSpeed:Number = 0;
		private var walking:Boolean = false;
		//private var speed:Number = 0;
		private var directionArrow:Bitmap;

		public function Attacker(world:World, player:MoosesPlayer) 
		{
			plate = new Nameplate(false);
			
			trace("player to create as attacker " + player);
			
			plate.SetName(player.getPlayerName());
			plate.SetHealth(100);

			gfx = new Attacker.GfxAttacker();
			gfx.x = -24;
			gfx.y = -24;
			
			directionArrow = new GfxAttackerArrow();
			
			addChild(directionArrow);
			addChild(gfx);
			addChild(plate);
			
			this.player = player;
			this.world = world;
			
			if (Main.DEBUG) {
				directionChange = getTimer();
				keyDown(Math.floor(Math.random() * 3 + 1) * 3);
			}
		}
		
		public function AddBuff(buff:LinkedListNode):void {
			buffs.insertAtEnd(buff);
		}
		
		public function AddHaste(value:Number):void {
			haste += value;
		}
		
		public function RemoveHaste(value:Number):void {
			haste -= value;
		}
		
		public function GetPlayer():MoosesPlayer {
			return player;
		}
		
		public function isReadyForRemoval():Boolean {
			return timeOfRemoval < getTimer();
		}
		public function getMoosesPlayer():MoosesPlayer {
				return player;
		}
		public function setMoosesPlayer(player:MoosesPlayer):void {
			
			this.player = player;
		}
		public function Update():void {
			
			
			
			if (IsAlive()) {
				
				if (walking && getTimer() > lastFrameSwap + 200) {
					
					lastFrameSwap = getTimer();
					if (lastFrame == 0) {
						lastFrame = 1;
						removeChild(gfx);
						gfx = new Attacker.GfxAttackerWalk1();
						gfx.x = -24;
						gfx.y = -24;
						addChild(gfx);
					} else {
						removeChild(gfx);
						lastFrame = 0;
						gfx = new Attacker.GfxAttackerWalk2();
						gfx.x = -24;
						gfx.y = -24;
						addChild(gfx);
					}
				}
				
				directionArrow.x = Math.cos(2 * Math.PI * (angle / 360) - Math.PI / 2) * 50;
				directionArrow.y = Math.sin(2 * Math.PI * (angle / 360) - Math.PI / 2) * 50;
				directionArrow.rotation = angle - 90;
				
				var b:LinkedListNode = buffs.firstNode as LinkedListNode;
				while (b != null) {
					IBuff(b).Update();
					b = b.nextNode;
				}
				

				if (Main.DEBUG && player.getPlayerId() > 9000 && getTimer() - directionChange > Math.random() * 200 + 200) {
					directionChange = getTimer();
					keyDown(Math.floor(Math.random() * 9) + 1);
				}
				
				
				var nX:Number;
				var nY:Number;
				angle += angleSpeed;
				if (world.GetGameState() == World.GS_RUNNING) {
					if (walking) {
						velocity = new Point(Math.cos( 2 * Math.PI * (angle/360) - Math.PI / 2), Math.sin(2 * Math.PI * (angle/360) - Math.PI / 2));
						trace(velocity.x + " , " + velocity.y);
						nX = x + velocity.x * 3 + haste * velocity.x;
						nY = y + velocity.y * 3 + haste * velocity.y;
						
						if (world.IsTraversable(nX, y)) {
							x = nX;
						}
						if (world.IsTraversable(x, nY)) {
							y = nY;
						}
					}
				}
				
				if (world.GetGameState() == World.GS_DEPLOY) {
					if (x > 400) {
						x = 400;
					}
				}
				if (x > World.WIDTH) {
					x = World.WIDTH;
				}
				if (y > World.HEIGHT) {
					y = World.HEIGHT;
				}
				if (x < 0) {
					x = 0;
				}
				if (y < 0) {
					y = 0;
				}
			} else {
				if (gfx.alpha > 0) {
					gfx.alpha = gfx.alpha - 0.03;
				}
			}
			//trace("Move attacker " + velocity + ", " + x + ", " + y);
		}
		
		public function Damage(damage:int):void {
			health -= damage;
			
			if (health <= 0) {
				Die();
			} else {
				plate.SetHealth(health);
			}
		}
		
		public function GetHealth():int {
			return health;
		}
		
		public function SetHealth(hp:int):void {
			health = hp;
			
			if (health <= 0) {
				Die();
			} else {
				plate.SetHealth(health);
			}
		}
		
		public function Die():void {
			
			if (alive) {
				timeOfDeath = getTimer();
				timeOfRemoval = getTimer() + 2000;
				alive = false;
				
				removeChild(gfx);
				removeChild(plate);
				
				gfx = new Attacker.GfxAttackerDead();
				gfx.x = -24;
				gfx.y = -24;
				
				addChild(gfx);
				
				velocity.x = 0; velocity.y = 0;
				walking = false;
				
				world.RespawnAttacker(this);
				
			}

		}
		
		public function IsAlive():Boolean {
			return alive;
		}
		public function Kill():void {
			alive = false;
			timeOfDeath = getTimer();
			timeOfRemoval = getTimer();
		}
		
		/*  a   b	|  12    13
		 *  1 2 3	|  01 02 03
		 *  4 5 6	|  04 05 06
		 *  7 8 9	|  07 08 09
		 *  * 0 #	|  10 00 11
		 */	
		public function keyUp(keyId:int):void {
			if (keyId == MoosesFrameworkClient.KEY_4) {
				angleSpeed = 0;
			} else if (keyId == MoosesFrameworkClient.KEY_6) {
				angleSpeed = 0;
			}
		}
		public function keyDown(keyId:int):void {
			
			//trace("key down, state " + world.GetGameState());
			if (alive) {
				
				
				if (keyId == MoosesFrameworkClient.KEY_5) {
					walking = true;
				} else if (keyId == MoosesFrameworkClient.KEY_4) {
					angleSpeed = -5;
				} else if (keyId == MoosesFrameworkClient.KEY_6) {
					angleSpeed = 5;
				} else if (keyId == MoosesFrameworkClient.KEY_2) {
					walking = true;
				} else if (keyId == MoosesFrameworkClient.KEY_8) {
					walking = false;
				}
				
				/*
				velocity.x = 0;
				velocity.y = 0;
				
				if (keyId == MoosesFrameworkClient.KEY_5) {
					velocity.x = 0;
					velocity.y = 0;
				} else if (keyId == MoosesFrameworkClient.KEY_4) {
					velocity.x = -1;
				} else if (keyId == MoosesFrameworkClient.KEY_6) {
					velocity.x = 1;
				} else if (keyId == MoosesFrameworkClient.KEY_2) {
					velocity.y = -1;
				} else if (keyId == MoosesFrameworkClient.KEY_8) {
					velocity.y = 1;
				} else if (keyId == MoosesFrameworkClient.KEY_1) {
					velocity.x = Math.cos(135);
					velocity.y = Math.sin(135);
				} else if (keyId == MoosesFrameworkClient.KEY_3) {
					velocity.x = Math.cos(45);
					velocity.y = Math.sin(45);
				} else if (keyId == MoosesFrameworkClient.KEY_7) {
					velocity.x = Math.cos(-135);
					velocity.y = Math.sin(-135);
				} else if (keyId == MoosesFrameworkClient.KEY_9) {
					velocity.x = Math.cos(-45);
					velocity.y = Math.sin(-45);
				}

				velocity.x = velocity.x * 2;
				velocity.y = velocity.y * 2;
				
				*/

			}
			
		}
		
		
		
		public function get previousNode():LinkedListNode {
			return _prevNode;
		}
		public function set previousNode(value:LinkedListNode):void {
			_prevNode = value;
		}
		
		public function get nextNode():LinkedListNode {
			return _nextNode;
		}
		public function set nextNode(value:LinkedListNode):void{
			_nextNode = value;
		}
		
		public function equals(o:Object):Boolean{
			return o === this;
		}
	}
	
}
