﻿package  no.ekse.mooses
{
	import flash.events.EventDispatcher;
	import no.ekse.mooses.MoosesFrameworkClient;
	
	/**
	 * This module maintains a list of players currently connected to the MOOSES framework.
	 * When a key event is found, this module sends the key event to the proper MoosesUnit.
	 * 
	 * @author Magnus Ekse
	 */
	public class MoosesData extends EventDispatcher
	{
		private var client:MoosesFrameworkClient;
		
		private var players:Array = new Array();
		
		public function MoosesData(client:MoosesFrameworkClient) 
		{
			
			this.client	= client;
			
			client.addEventListener(MoosesPlayerEvent.MOOSES_PLAYER_JOIN, playerJoin);
			client.addEventListener(MoosesPlayerEvent.MOOSES_PLAYER_QUIT, playerQuit);
			
			client.addEventListener(MoosesKeyEvent.MOOSES_KEY_EVENT, keyEvent);
			
		}
		
		public function getPlayerList():Array {
			
			return players;
			
		}
		
		public function buzzUnit(unitToBuzz:MoosesUnit, duration:int):void {
			
			for (var i:int; i < players.length; i++) {
				if (players[i].getAttachedUnit() == unitToBuzz) {
					client.buzzPlayer(players[i].getPlayerId(), duration);
				}
			}
			
		}
		
		public function addPlayer(id:int, name:String):void {
			
			players.push(new MoosesPlayer(this, id, name));
			
		}
		
		public function updatePlayerScore(playerId:int, score:int):void {
			
			client.updatePlayerScore(playerId, score);
		}
		public function sendMessageToPlayer(playerId:int, messageType:int, messageData:Array):void {
			
			client.sendMessageToPlayer(playerId, messageType, messageData);
			
		}
		
		public function broadcastMessage(messageType:int, messageData:Array):void {
			
			for (var i:int; i < players.length; i++) {
				client.sendMessageToPlayer( (players[i] as MoosesPlayer).getPlayerId(), messageType, messageData);
			}
			
		}
		
		public function removePlayer(id:int):void {
			
			var playerId:int = id;
			var playerToRemoveIndex:int;
			
			for (var i:int; i < players.length; i++) {
				if (players[i].getPlayerId() == playerId) {
					playerToRemoveIndex = i;
				}
			}
			
			players.splice(playerToRemoveIndex, 1);
			
		}
		
		public function playerJoin(e:MoosesPlayerEvent):void {
			
			addPlayer(e.playerId, e.playerName);
			
		}
		public function playerQuit(e:MoosesPlayerEvent):void {
			
			removePlayer(e.playerId);
			
		}
		
		public function keyEvent(e:MoosesKeyEvent):void {
			
			for each (var player:MoosesPlayer in players) {
				
				if (player.getPlayerId() == e.playerId) {
					
					
					if (e.keyState == MoosesFrameworkClient.KEY_STATE_DOWN) {
						//State = 0, pressed.
						player.keyDown(e.keyCode);
					} else if (e.keyState == MoosesFrameworkClient.KEY_STATE_UP) {
						//State = 1, released.
						player.keyUp(e.keyCode);
					} else if (e.keyState == MoosesFrameworkClient.KEY_STATE_SPECIAL) {
						//State = 2, pressed and will not be released.
						player.keyDown(e.keyCode);
					}
					
					
				}
			}
			
		}
		
	}
	
}